use strict;
use warnings;

my ($in_metaeuk_reps_to_min_seqid_file, $in_metaeuk_reps_vs_annot_prots_m8_file, $in_metaeuk_reps_fasta_file, $in_annot_protein_coords_file, 
	$in_annot_non_coding_gene_features_file, $out_metaeuk_annot_prot_report_file, $out_metaeuk_mapping_all_file) = @ARGV;

my %metaeuk_reps_to_info;
parse_metaeuk_styled_headers($in_metaeuk_reps_fasta_file, -1, \%metaeuk_reps_to_info);

my %contigs_with_annot;
my %annot_prots_to_loc;
parse_metaeuk_styled_headers($in_annot_protein_coords_file, 0, \%annot_prots_to_loc, \%contigs_with_annot);

my %annot_noncoding_to_loc;
parse_metaeuk_styled_headers($in_annot_non_coding_gene_features_file, 0, \%annot_noncoding_to_loc, \%contigs_with_annot);

match_pred_with_annot_prots($in_metaeuk_reps_vs_annot_prots_m8_file, \%metaeuk_reps_to_info, \%annot_prots_to_loc);

match_pred_w_non_coding_or_opposite_strand_prot(\%metaeuk_reps_to_info, \%annot_prots_to_loc, \%annot_noncoding_to_loc);

match_pred_with_min_seqid($in_metaeuk_reps_to_min_seqid_file, \%metaeuk_reps_to_info);

open (my $out, ">", $out_metaeuk_annot_prot_report_file) or die "could not open $out_metaeuk_annot_prot_report_file for writing\n";
print $out "T\tC\tS\tevalue\tmin_seqid_group\tannotProt\tannot_cov\tnum_shared_CDSs\tnum_covered_CDSs_metaeuk\tnum_covered_CDSs_annot\tnum_CDSs_metaeuk\tnum_CDSs_annot\n";
foreach my $metaeuk_TCS (keys %metaeuk_reps_to_info) {
	
	my $metaeuk_target = $metaeuk_reps_to_info{$metaeuk_TCS}{"target"};
	my $metaeuk_contig = $metaeuk_reps_to_info{$metaeuk_TCS}{"contig"};
	my $metaeuk_strand = $metaeuk_reps_to_info{$metaeuk_TCS}{"strand"};
	my $metaeuk_evale = $metaeuk_reps_to_info{$metaeuk_TCS}{"evalue"};
	
	if (! exists $metaeuk_reps_to_info{$metaeuk_TCS}{"min_seqid_group"}) {
		die "Error: could not find group inforamtion for $metaeuk_TCS";
	}
	my $min_group_seqid = $metaeuk_reps_to_info{$metaeuk_TCS}{"min_seqid_group"};
	my $metaeuk_num_CDSs = $metaeuk_reps_to_info{$metaeuk_TCS}{"num_exons"};
	
	if (exists $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prot_in_loc"}) {
		foreach my $annot_prot_in_loc_ref (@{$metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prot_in_loc"}}) {
			my $annotProt = $annot_prot_in_loc_ref->{"annot_prot_id"};
			my $annot_cov = $annot_prot_in_loc_ref->{"annot_cov"};
			my $num_shared_CDSs = $annot_prot_in_loc_ref->{"num_shared_CDSs"};
			my $num_covered_CDSs_metaeuk = $annot_prot_in_loc_ref->{"num_covered_CDSs_metaeuk"};
			my $num_covered_CDSs_annot = $annot_prot_in_loc_ref->{"num_covered_CDSs_annot"};
			my $num_CDSs_annot = $annot_prot_in_loc_ref->{"num_CDSs_annot"};
			
			print $out "$metaeuk_target\t$metaeuk_contig\t$metaeuk_strand\t$metaeuk_evale\t$min_group_seqid\t";
			print $out "$annotProt\t$annot_cov\t$num_shared_CDSs\t$num_covered_CDSs_metaeuk\t$num_covered_CDSs_annot\t";
			print $out "$metaeuk_num_CDSs\t$num_CDSs_annot\n";
		}
	}
}
close ($out);


open (my $out2, ">", $out_metaeuk_mapping_all_file) or die "could not open $out_metaeuk_mapping_all_file for writing\n";
print $out2 "T\tC\tS\tevalue\tType\tannotFeature\tpropOverlap\n";
foreach my $metaeuk_TCS (keys %metaeuk_reps_to_info) {
	
	my $metaeuk_target = $metaeuk_reps_to_info{$metaeuk_TCS}{"target"};
	my $metaeuk_contig = $metaeuk_reps_to_info{$metaeuk_TCS}{"contig"};
	my $metaeuk_strand = $metaeuk_reps_to_info{$metaeuk_TCS}{"strand"};
	my $metaeuk_evalue = $metaeuk_reps_to_info{$metaeuk_TCS}{"evalue"};
	print $out2 "$metaeuk_target\t$metaeuk_contig\t$metaeuk_strand\t$metaeuk_evalue\t";
	
	if (exists $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prot_in_loc"}) {
		print $out2 "mapped_to_prot\tNA\tNA\n";
	} elsif (exists $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_noncoding_in_loc"}) {
		my $annotNonCoding = $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_noncoding_in_loc"}{"annot_noncoding"};
		my $prop_overlap = $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_noncoding_in_loc"}{"prop_overlap"};
		print $out2 "non_coding\t$annotNonCoding\t$prop_overlap\n";
	} elsif (exists $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prots_opp_strand"}) {
		my $annotOtherStrand = $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prots_opp_strand"}{"annot_prot_id"};
		my $prop_overlap = $metaeuk_reps_to_info{$metaeuk_TCS}{"annot_prots_opp_strand"}{"prop_overlap"};
		print $out2 "opp_strand\t$annotOtherStrand\t$prop_overlap\n";
	} elsif (! exists $contigs_with_annot{$metaeuk_contig}) {
		print $out2 "unannot_contig\tNA\tNA\n";
	}
	else {
		print $out2 "unmapped\tNA\tNA\n";
	}
}
close ($out2);

sub match_pred_with_min_seqid {
	my ($in_metaeuk_reps_to_min_seqid_file, $metaeuk_reps_to_info_ref) = @_;
	open (my $in, "<", $in_metaeuk_reps_to_min_seqid_file) or die "could not open $in_metaeuk_reps_to_min_seqid_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		#UniRef90_A0A182L8Q5|NC_003421.2|-	0.525
		#UniRef90_V4BHB9|NC_003424.3|-	0.617
		chomp ($line);
		my ($TCS_rep, $min_group_seqid) = split(/\t/, $line);
		
		if (exists $metaeuk_reps_to_info_ref->{$TCS_rep}) {
			$metaeuk_reps_to_info_ref->{$TCS_rep}{"min_seqid_group"} = $min_group_seqid;
		}
		
		$line = <$in>;
	}
	close ($in);
}

sub match_pred_w_non_coding_or_opposite_strand_prot {
	my ($metaeuk_reps_to_info_ref, $annot_prots_to_loc_ref, $annot_noncoding_to_loc_ref) = @_;
	
	# go over all preds - for those which don't match a loc of a protein, check if they match a noncoding feature or an annotated protein on the opposite strand
	foreach my $metaeuk_TCS (keys (%{$metaeuk_reps_to_info_ref})) {
		if (! exists $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prot_in_loc"}) {
			my $metaeuk_contig = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"contig"};
			my $metaeuk_strand = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"strand"};
			my $metaeuk_low_coord = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"low_coord"};
			my $metaeuk_high_coord = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"high_coord"};
			
			# check if overlaps an annotated non coding feature
			foreach my $noncoding_feature (keys (%{$annot_noncoding_to_loc_ref})) {
				my $annot_noncoding_contig = $annot_noncoding_to_loc_ref->{$noncoding_feature}{"contig"};
				my $annot_noncoding_strand = $annot_noncoding_to_loc_ref->{$noncoding_feature}{"strand"};
				my $annot_noncoding_low_coord = $annot_noncoding_to_loc_ref->{$noncoding_feature}{"low_coord"} - 1;
				my $annot_noncoding_high_coord = $annot_noncoding_to_loc_ref->{$noncoding_feature}{"high_coord"} - 1;
				
				if (($metaeuk_contig eq $annot_noncoding_contig) and ($metaeuk_strand eq $annot_noncoding_strand)) {
					my ($metaeuk_length, $annot_noncoding_length, $num_chars_overlap) = check_four_coords($metaeuk_low_coord, $metaeuk_high_coord, $annot_noncoding_low_coord, $annot_noncoding_high_coord);
					
					# if at least 10 chars overlap to an annotation of a nocoding feature
					if ($num_chars_overlap > 10) {
						if ((! exists $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_noncoding_in_loc"}) or 
							($metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_noncoding_in_loc"}{"prop_overlap"} < ($num_chars_overlap / $annot_noncoding_length))) {	
							$metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_noncoding_in_loc"} = {"annot_noncoding" => $noncoding_feature, "prop_overlap" => ($num_chars_overlap / $annot_noncoding_length)};
						}
					}	
				}
			}
			
			# check if overlaps an annotated protein on the opposite strand
			foreach my $annot_prot_id (keys (%{$annot_prots_to_loc_ref})) {
				my $annot_prot_contig = $annot_prots_to_loc_ref->{$annot_prot_id}{"contig"};
				my $annot_prot_strand = $annot_prots_to_loc_ref->{$annot_prot_id}{"strand"};
				my $annot_prot_low_coding = $annot_prots_to_loc_ref->{$annot_prot_id}{"low_coding_coord"} - 1; # to fit base-0 of MetaEuk
				my $annot_prot_high_coding = $annot_prots_to_loc_ref->{$annot_prot_id}{"high_coding_coord"} - 1; # to fit base-0 of MetaEuk
				
				if (($metaeuk_contig eq $annot_prot_contig) and ($metaeuk_strand ne $annot_prot_strand)) {
					my ($metaeuk_length, $annot_prot_length, $num_chars_overlap) = check_four_coords($metaeuk_low_coord, $metaeuk_high_coord, $annot_prot_low_coding, $annot_prot_high_coding);
					
					# if at least 10 chars overlap to an annotation of a nocoding feature
					if ($num_chars_overlap > 10) {
						if ((! exists $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prots_opp_strand"}) or 
							($metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prots_opp_strand"}{"prop_overlap"} < ($num_chars_overlap / $annot_prot_length))) {	
							$metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prots_opp_strand"} = {"annot_prot_id" => $annot_prot_id, "prop_overlap" => ($num_chars_overlap / $annot_prot_length)};
						}
					}	
				}
			}
		}
	}
}

sub match_pred_with_annot_prots {
	my ($in_metaeuk_reps_vs_annot_prots_m8_file, $metaeuk_reps_to_info_ref, $annot_prots_to_loc_ref) = @_;
	
	open (my $in, "<", $in_metaeuk_reps_vs_annot_prots_m8_file) or die "could not open $in_metaeuk_reps_vs_annot_prots_m8_file for reading\n";
	my $m8_line = <$in>;
	while (defined $m8_line)
	{
		chomp($m8_line);
		# UniRef90_L8GDC2|NW_004457244.1|-|500|1.45e-140|1|133735|134463|134463[134463]:133735[133735]:729[729]	XP_004338291.1	1.523E-132	0.867	0.996	0.938	242	16	4
		my ($metaeuk_pred_header, $annot_prot_id, $evalue, $pident, $qcov, $tcov, $alnlen, $mismatch, $gapopen) = split(/\t/, $m8_line);
		my $metaeuk_TCS = get_TCS_key($metaeuk_pred_header);
		
		# sanity check
		if (! exists $metaeuk_reps_to_info_ref->{$metaeuk_TCS}) {
			die "Error: '$metaeuk_TCS' was not found in MetaEuk's results\n";
		}
		# sanity check
		if (! exists $annot_prots_to_loc_ref->{$annot_prot_id}) {
			die "Error: '$annot_prot_id' was not found in annotated proteins coords results\n";
		}
		
		my $metaeuk_contig = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"contig"};
		my $metaeuk_strand = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"strand"};
		
		my $annot_prot_contig = $annot_prots_to_loc_ref->{$annot_prot_id}{"contig"};
		my $annot_prot_strand = $annot_prots_to_loc_ref->{$annot_prot_id}{"strand"};
		
		my $curr_annot_prot_overlaps = "false";
		
		# if on the same contig and strand - check coords
		if (($metaeuk_contig eq $annot_prot_contig) and ($metaeuk_strand eq $annot_prot_strand)) {
			my $metaeuk_low_coding = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"low_coding_coord"};
			my $metaeuk_high_coding = $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"high_coding_coord"};
			
			my $annot_prot_low_coding = $annot_prots_to_loc_ref->{$annot_prot_id}{"low_coding_coord"} - 1; # to fit base-0 of MetaEuk
			my $annot_prot_high_coding = $annot_prots_to_loc_ref->{$annot_prot_id}{"high_coding_coord"} - 1; # to fit base-0 of MetaEuk
			
			my ($metaeuk_length, $annot_prot_length, $num_chars_overlap) = check_four_coords($metaeuk_low_coding, $metaeuk_high_coding, $annot_prot_low_coding, $annot_prot_high_coding);
			
			# if 80% of the prediction or of the annotated protein is covered by the overlap
			# AND mismatches (i.e., non-identical and non-gaps) count for less than 10% of the alignment
			
			if (((($num_chars_overlap / $metaeuk_length) > 0.8) or (($num_chars_overlap / $annot_prot_length) > 0.8)) and (($mismatch / $alnlen) < 0.1)) {
				$curr_annot_prot_overlaps = "true";
				
				# compare CDSs array
				my ($num_shared_CDSs, $num_covered_CDSs_metaeuk, $num_covered_CDSs_annot, $num_CDSs_metaeuk, $num_CDSs_annot) = compare_CDSs($metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"CDSs_info"}, $annot_prots_to_loc_ref->{$annot_prot_id}{"CDSs_info"});
				
				# sanity check
				if ($num_CDSs_metaeuk != $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"num_exons"}) {
					die "Error - num exons doesn't match: $num_CDSs_metaeuk and $metaeuk_TCS\n";
				}
				
				# insert mapped annot prot to array
				if (! exists $metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prot_in_loc"}) {
					$metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prot_in_loc"} = [{"annot_prot_id" => $annot_prot_id, "evalue" => $evalue, "seq_id" => $pident, 
																							   "prop_mismatch" => ($mismatch / $alnlen), "pred_cov" => $qcov, "annot_cov" => $tcov,
																							   "num_shared_CDSs" => $num_shared_CDSs, "num_covered_CDSs_metaeuk" => $num_covered_CDSs_metaeuk,
																							   "num_covered_CDSs_annot" => $num_covered_CDSs_annot, "num_CDSs_annot" => $num_CDSs_annot}];
				} else {
					push (@{$metaeuk_reps_to_info_ref->{$metaeuk_TCS}{"annot_prot_in_loc"}}, {"annot_prot_id" => $annot_prot_id, "evalue" => $evalue, "seq_id" => $pident, 
																							   		  "prop_mismatch" => ($mismatch / $alnlen), "pred_cov" => $qcov, "annot_cov" => $tcov,
																							   		  "num_shared_CDSs" => $num_shared_CDSs, "num_covered_CDSs_metaeuk" => $num_covered_CDSs_metaeuk,
																							   		  "num_covered_CDSs_annot" => $num_covered_CDSs_annot, "num_CDSs_annot" => $num_CDSs_annot});
				}
			}
		}
		$m8_line = <$in>;
	}
	close ($in);
}

sub compare_CDSs {
	# this sub is called on a MetaEuk prediction and its mapped annotated protein
	my ($metaeuk_CDSs_array_ref, $annot_prot_CDSs_array_ref) = @_;

	my $num_metaeuk_CDSs = @{$metaeuk_CDSs_array_ref};
	my $num_annot_prot_CDSs = @{$annot_prot_CDSs_array_ref};
	
	my $num_shared_CDSs = 0; # cover both >80% of metaeuk CDS and >80% of annot_prot CDS
	my $num_covered_CDSs_metaeuk = 0; # cover only >80% of metaeuk CDS
	my $num_covered_CDSs_annot = 0; # cover only >80% of annot_prot CDS
	
	foreach my $metaeuk_CDS_hash_ref (@{$metaeuk_CDSs_array_ref}) {
		my $metaeuk_low_CDS = $metaeuk_CDS_hash_ref->{"CDS_low"};
		my $metaeuk_high_CDS = $metaeuk_CDS_hash_ref->{"CDS_high"};
		foreach my $annot_prot_CDS_hash_ref (@{$annot_prot_CDSs_array_ref}) {
			my $annot_prot_low_CDS = $annot_prot_CDS_hash_ref->{"CDS_low"} - 1;
			my $annot_prot_high_CDS = $annot_prot_CDS_hash_ref->{"CDS_high"} - 1;
			my ($metaeuk_CDS_length, $annot_prot_CDS_length, $num_chars_overlap) = check_four_coords($metaeuk_low_CDS, $metaeuk_high_CDS, $annot_prot_low_CDS, $annot_prot_high_CDS);
			
			my $metaeuk_CDS_cover = $num_chars_overlap / $metaeuk_CDS_length;
			my $annot_prot_CDS_cover = $num_chars_overlap / $annot_prot_CDS_length;
			
			if (($metaeuk_CDS_cover > 0.8) and ($annot_prot_CDS_cover > 0.8)) {
				$num_shared_CDSs++;
				$metaeuk_CDS_hash_ref->{"status"} = "shared";
				# can go to outer loop
				last;
			} elsif ($metaeuk_CDS_cover > 0.8) {
				$num_covered_CDSs_metaeuk++;
				$metaeuk_CDS_hash_ref->{"status"} = "covered";
				# can go to outer loop
				last;
			} elsif ($annot_prot_CDS_cover > 0.8) {
				$metaeuk_CDS_hash_ref->{"status"} = "covering";
				$num_covered_CDSs_annot++;
			}
		}
	}
	
	return ($num_shared_CDSs, $num_covered_CDSs_metaeuk, $num_covered_CDSs_annot, $num_metaeuk_CDSs, $num_annot_prot_CDSs);	
}

sub check_four_coords {
	my ($start_1, $end_1, $start_2, $end_2) = @_;
	
	my $length_1 = ($end_1 - $start_1 + 1);
	my $length_2 = ($end_2 - $start_2 + 1);
	
	if (($length_1 < 0) or ($length_2 < 0)) {
		die "Error: assume start_pos is <= end_pos\n";
	}
	
	my $num_chars_overlap = 0;
	## two ways to get an overlap
	# S1 <= S2 < E1
	if (($start_1 <= $start_2) and ($start_2 < $end_1)) {
		if ($end_1 < $end_2) {
			$num_chars_overlap = ($end_1 - $start_2 + 1);
		} else {
			$num_chars_overlap = ($end_2 - $start_2 + 1);
		}	
	}
	# S2 < S1 < E2
	if (($start_2 < $start_1) and ($start_1 < $end_2)) {
		if ($end_2 < $end_1) {
			$num_chars_overlap = ($end_2 - $start_1 + 1);
		} else {
			$num_chars_overlap = ($end_1 - $start_1 + 1);
		}
	}
	
	if ($num_chars_overlap < 0) {
		die "Error: something is wrong with the computation: $start_1, $end_1, $start_2, $end_2\n";
	}
	
	return ($length_1, $length_2, $num_chars_overlap);
}

sub parse_metaeuk_styled_headers {
	my ($in_fasta_file, $column_num_as_key, $elements_to_loc_hash_ref, $contigs_with_annot_ref) = @_;
	open (my $in, "<", $in_fasta_file) or die "could not open $in_fasta_file for reading\n";
	my $line = <$in>;
	while (defined $line) {
		chomp $line;
		if ($line =~ m/^>(.*)/) {
			my $header = $1;
			my @header_parts = split(/\|/, $header);
			my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = @header_parts;
			
			# for annotations - keep track of annotated contigs
			if (defined $contigs_with_annot_ref) {
				$contigs_with_annot_ref->{$contig_id} = 1;
			}
			
			my $key = $header;
			if ($column_num_as_key > -1) {
				$key = $header_parts[$column_num_as_key];
			} elsif ($column_num_as_key == -1) {
				$key = get_TCS_key($header);
			}
			
			# allow chopped scaffolds analysis:
			my $chop_offset = 0;
			my @contig_id_parts = split(/\;/, $contig_id);
			if (@contig_id_parts == 3) {
				$contig_id = $contig_id_parts[0];
				$chop_offset = $contig_id_parts[1];
			}
			$low_coord += $chop_offset;
			$high_coord += $chop_offset;
			
			my $lowest_coding = "NA";
			my $highest_coding = "NA";
			my $is_first_iter = "true";
			my @CDSs_info = ();
			foreach my $exon_info (@exons_info) {
				my ($coord_1_str, $coord_2_str, $length) = split (/\:/, $exon_info);
				my $coord_1 = "";
				if ($coord_1_str =~ m/\[(\d+)\]/) {
					$coord_1 = $1;
				}
				my $coord_2 = "";
				if ($coord_2_str =~ m/\[(\d+)\]/) {
					$coord_2 = $1;
				}
				
				my $CDS_low = $coord_1;
				my $CDS_high = $coord_2;
				if ($coord_1 > $coord_2) {
					$CDS_low = $coord_2;
					$CDS_high = $coord_1;	
				}
				
				# support chopped analysis:
				$CDS_low += $chop_offset;
				$CDS_high += $chop_offset;
				
				push (@CDSs_info, {"CDS_low" => $CDS_low, "CDS_high" => $CDS_high});
				
				# record the lowest and highest coding coords
				if ($is_first_iter eq "true") {
					$lowest_coding = $CDS_low;
					$highest_coding = $CDS_high;
					$is_first_iter = "false";
				} else {
					if ($lowest_coding > $CDS_low) {
						$lowest_coding = $CDS_low;
					}
					if ($highest_coding < $CDS_high) {
						$highest_coding = $CDS_high;
					}
				}
				
			}
			
			$elements_to_loc_hash_ref->{$key} = {"target" => $target_id, "contig" => $contig_id, "strand" => $strand, "bitscore" => $sum_bit_score, "evalue" => $evalue, 
													"num_exons" => $num_exons, "low_coord" => $low_coord, "high_coord" => $high_coord, "CDSs_info" => [@CDSs_info],
													"low_coding_coord" => $lowest_coding, "high_coding_coord" => $highest_coding};
		}
		$line = <$in>;
	}
	close ($in);
}

sub get_TCS_key {
	my ($header) = @_;
	my @header_parts = split(/\|/, $header);
	my ($target_id, $contig_id, $strand, $sum_bit_score, $evalue, $num_exons, $low_coord, $high_coord, @exons_info) = @header_parts;
	my $TCS_key = join("\|", ($target_id, $contig_id, $strand));
	return($TCS_key);
}

