use strict;
use warnings;

my ($in_fasta_file, $out_fasta_file, $mode) = @ARGV;

open (my $out, ">", $out_fasta_file) or die "could not open $out_fasta_file for reading";
open (my $in, "<", $in_fasta_file) or die "could not open $in_fasta_file for reading";
my $line = <$in>;

while (defined $line) {
	my $line_to_write = $line;
	if ($line =~ m/^>/) {
		if ((defined $mode) and ($mode eq "uniclust")) {
			$line_to_write = shorten_header_uniclust($line);
		}
		elsif ((defined $mode) and ($mode eq "mmetsp")) {
			$line_to_write = shorten_header_mmetsp($line);
		}
		else {
			$line_to_write = shorten_header_simple($line);
		}
	}
	print $out "$line_to_write";
	$line = <$in>;
}

close ($in);
close ($out);

sub shorten_header_simple
{
	my ($orig_header) = @_;
	my $short_header = "";
	if ($orig_header =~ m/^>(\S+)\s/) {
		$short_header = ">$1\n";
	}
	return($short_header);
}

sub shorten_header_uniclust
{
	my ($orig_header) = @_;
	#>sp|Q91G88|006L_IIV6 Putative KilA-N domain-containing protein 006L OS=Invertebrate iridescent virus 6 GN=IIV6-006L PE=3 SV=1
	my ($junk, $short_header) = split(/\|/, $orig_header);
	$short_header = ">$short_header\n";
	return($short_header);
}

sub shorten_header_mmetsp
{
	my ($orig_header) = @_;
	# >MMETSP0905-doi:10.5281/zenodo.249982-MMETSP0905_Transcript_0|m.1 MMETSP0905_Transcript_0|g.1  ORF MMETSP0905_Transcript_0|g.1 MMETSP0905_Transcript_0|m.1 type:complete len:217 (-) MMETSP0905_Transcript_0:405-1055(-)
	# >MMETSP1143-doi:10.5281/zenodo.249982-Gene.122::MMETSP1143_Transcript_100::g.122::m.122 Gene.122::MMETSP1143_Transcript_100::g.122  ORF type:internal len:211 (+) MMETSP1143_Transcript_100:2-631(+)
	my $short_header = "";
	if ($orig_header =~ m/(MMETSP\d+\_Transcript_\d+)/) {
		$short_header = $1;
	}
	else {
		die "NOT IN FORMAT: $orig_header\n";
	}
	$short_header = ">$short_header\n";
	return($short_header);
}