#!/bin/bash -ex
#BSUB -J getAndAssemble[1-912]%30
#BSUB -q mpi-long+
#BSUB -o out.%I.%J
#BSUB -e err.%I.%J
#BSUB -W 120:00
#BSUB -m hh
#BSUB -n 16
#BSUB -R haswell
#BSUB -R cbscratch
#BSUB -R "span[ptile=16]"
MEGAHIT_PATH="megahit"
TRIMMOATIC_PATH="trimmomatic-0.38.jar"
FASTQ_DUMP="sratoolkit.2.9.1-1-ubuntu64/bin/fastq-dump"
OUTPATH="YOUR_DIR/assembled_atlas"
id="$(sed "${LSB_JOBINDEX}q;d" YOUR_DIR/TARA/TARA_MetaG_ERRs.txt)"
mkdir -p "/local_hdd/$id"
cd "/local_hdd/$id"

function finish {
	rm -rf "/local_hdd/$id"
}
trap finish EXIT

if [  ! -f "$OUTPATH/${id}_megahit_final.contigs.fa.gz" ]
then
	wget "https://sra-download.ncbi.nlm.nih.gov/srapub/${id}"
	
	"${FASTQ_DUMP}" --split-files -split-3 "/local_hdd/$id/$id"

	rm "$id"
	
	# Trimmomatic and its parameters:
	# Remove adapters (ILLUMINACLIP:TruSeq3-PE.fa:2:30:10)
	# Remove leading low quality or N bases (below quality 3) (LEADING:3)
	# Remove trailing low quality or N bases (below quality 3) (TRAILING:3)
	# Scan the read with a 4-base wide sliding window, cutting when the average quality per base drops below 15 (SLIDINGWINDOW:4:15)
	# Drop reads below the 36 bases long (MINLEN:36)
	
	if [ -f "${id}_1.fastq" ]
	then
		# Paired-End:
		java -jar "${TRIMMOATIC_PATH}" PE -threads 16 -phred33 "${id}_1.fastq" "${id}_2.fastq" "${id}_1_paired_output.fastq" "${id}_1_unpaired_output.fastq" "${id}_2_paired_output.fastq" "${id}_2_unpaired_output.fastq" TruSeq3-PE.fa:2:30:10 LEADING:3 TRAILING:3 SLIDINGWINDOW:4:15 MINLEN:36
		
		rm "${id}_1.fastq" "${id}_2.fastq"
		
		# Megahit and its parameters (all in default, retaining contigs of at least 200bp):
		
		"${MEGAHIT_PATH}" --num-cpu-threads 16 --min-contig-len 200 -1 "${id}_1_paired_output.fastq" -2 "${id}_2_paired_output.fastq" -r "${id}_1_unpaired_output.fastq","${id}_2_unpaired_output.fastq" -o "${id}_megahit"
		
		rm "${id}_1_paired_output.fastq" "${id}_2_paired_output.fastq" "${id}_1_unpaired_output.fastq" "${id}_2_unpaired_output.fastq"
		
		touch "$OUTPATH/${id}_PE.txt"
	else
		# Single-End:
		java -jar "${TRIMMOATIC_PATH}" SE -threads 16 -phred33 "${id}.fastq" "${id}_output.fastq" TruSeq3-SE.fa:2:30:10 LEADING:3 TRAILING:3 SLIDINGWINDOW:4:15 MINLEN:36
		
		rm "${id}.fastq"
		
		# Megahit and its parameters (all in default, retaining contigs of at least 200bp):
		
		"${MEGAHIT_PATH}" --num-cpu-threads 16 --min-contig-len 200 -r "${id}_output.fastq" -o "${id}_megahit"
		
		rm "${id}_output.fastq"
		
		touch "$OUTPATH/${id}_SE.txt"
		
	fi

	gzip -c "${id}_megahit/final.contigs.fa" > "$OUTPATH/${id}_megahit_final.contigs.fa.gz"
	
	rm -rf "${id}_megahit"
	
fi