#!/bin/bash -ex
#SBATCH -J TaraPreds_vs_T_EUK_REFS
#SBATCH -p hh
#SBATCH -o out.%J
#SBATCH -n 160
#SBATCH -N 10
#SBATCH -t 14-00:00:00
export RUNNER="mpirun --pernode --bind-to none"

MMSEQS_PATH="YOUR_DIR/mmseqs2_790d/MMseqs2/build/src/mmseqs"
SCRIPTS_DIR="YOUR_DIR/scripts"
METAEUK_RESULTS_OUTDIR="TaraEukRep5kb_EUK_profiles_011"
METAEUK_PREDS_FASTA="${METAEUK_RESULTS_OUTDIR}/predRed_Tara5kbEukRep.fas"

OUTDIR="${METAEUK_RESULTS_OUTDIR}/Preds_vs_T_EUK_REFS"
MERC_MMETSP_UNICLUST50="SEE_RELEASED_DATA/MMETSP_uniclust50_MERC"

mkdir -p "${OUTDIR}"
cd "${OUTDIR}"

# createdb
if [ ! -f "${OUTDIR}/predRed.dbtype" ]
then
	"${MMSEQS_PATH}" createdb "${METAEUK_PREDS_FASTA}" predRed --dbtype 1
fi

# create subdb of only the targets that contributed a prediction
if [ ! -f "${OUTDIR}/target_euk_refs.dbtype" ]
then
	# write id and header for each of the EUK_REFs
	"${MMSEQS_PATH}" prefixid "${MERC_MMETSP_UNICLUST50}_h" euk_refs_h.tsv --tsv

	# obtain a list of unique EUK_REFs headers used for the MetaEuk predictions
	cut -d'|' -f1 predRed_h | sort | uniq > targets_of_predRed_h.txt

	# move the null bytes
	tr < targets_of_predRed_h.txt -d '\000' > targets_of_predRed_h2.txt
	# first line is an empty line...
	tail -n +2 targets_of_predRed_h2.txt > targets_of_predRed_h.txt

	# get a tsv of id and header only for EUK_REFs that served as a MetaEuk target
	grep -F -w -f targets_of_predRed_h.txt euk_refs_h.tsv > target_euk_refs_h.tsv
	
	# grep missed one for some reason, had to manually add it.

	# create a subdb of all EUK_REFs that served as a MetaEuk target
	"${MMSEQS_PATH}" createsubdb target_euk_refs_h.tsv "${MERC_MMETSP_UNICLUST50}" target_euk_refs
fi

# search preds vs EUK_REFS
if [ ! -f "${OUTDIR}/predRed_vs_T_EUK_REFS.dbtype" ]
then
	"${MMSEQS_PATH}" search predRed target_euk_refs predRed_vs_T_EUK_REFS tmp -a -s 7 -e 1
fi

if [ ! -f "${OUTDIR}/predRed_vs_T_EUK_REFS.m8" ]
then
	"${MMSEQS_PATH}" convertalis predRed target_euk_refs predRed_vs_T_EUK_REFS predRed_vs_T_EUK_REFS.m8 --format-output query,target,pident,alnlen,mismatch,gapopen,qstart,qend,tstart,tend,evalue,bits,qlen,tlen,qcov,tcov
fi

if [ ! -f "${OUTDIR}/predRed_vs_EUK_REFS_ownT.m8" ]
then
	perl "${SCRIPTS_DIR}/extract_vs_own_target.pl" predRed_vs_T_EUK_REFS.m8 predRed_vs_EUK_REFS_ownT.m8
	perl "${SCRIPTS_DIR}/m8_to_R_cols.pl" predRed_vs_EUK_REFS_ownT.m8 predRed_vs_EUK_REFS_ownT_some_cols.tsv
fi


