use strict;
use warnings;

my ($in_metaeuk_preds_fasta, $in_mmetsp_to_taxinfo, $in_preds_best_uniref90_taxinfo_tsv, $in_preds_best_mmetsp_taxinfo_tsv, $out_metaeuk_preds_to_tax, $out_contigs_to_tax, $out_contigs_to_tax_for_corona) = @ARGV;

my %mmetsp_to_tax_info;
my %contig_to_preds;

get_mmetsp_to_tax_info($in_mmetsp_to_taxinfo, \%mmetsp_to_tax_info);
parse_metaeuk_fasta($in_metaeuk_preds_fasta, \%mmetsp_to_tax_info, \%contig_to_preds);
add_best_hit_info($in_preds_best_uniref90_taxinfo_tsv, "SOURCE_UNICLUST90", \%contig_to_preds);
add_best_hit_info($in_preds_best_mmetsp_taxinfo_tsv, "SOURCE_MMETSP", \%contig_to_preds);
write_best_pred_and_contig(\%contig_to_preds,  $out_metaeuk_preds_to_tax, $out_contigs_to_tax);

sub write_best_pred_and_contig
{
	my ($contig_to_preds_ref, $out_metaeuk_preds_to_tax, $out_contigs_to_tax) = @_;
	
	open (my $out_preds, ">", $out_metaeuk_preds_to_tax) or die "could not open $out_metaeuk_preds_to_tax for writing\n";
	print $out_preds "HEADER\tBITSCORE_BEST_MATCH\tEVALUE_BEST_MATCH\tSOURCE\tTAX_ACC\tSEQID_TO_TAX\tTAX_LEVEL\tLINEAGE_INFO\n";
	
	open (my $out_cont, ">", $out_contigs_to_tax) or die "could not open $out_contigs_to_tax for writing\n";
	print $out_cont "CONTIG\tBEST_HEADER\tBITSCORE_BEST_MATCH\tEVALUE_BEST_MATCH\tSOURCE\tTAX_ACC\tSEQID_TO_TAX\tTAX_LEVEL\tLINEAGE_INFO\tNUM_PREDS\n";
	
	open (my $out_corona, ">", $out_contigs_to_tax_for_corona) or die "could not open $out_contigs_to_tax_for_corona for writing\n";
	print $out_corona "NUM\tTAX_ID\tTAX_RANK\tSEQID_TO_TAX\tCONTIG\n";
	
	my $num = 0;
	
	foreach my $contig (keys %{$contig_to_preds_ref})
	{
		my $best_bitscore = 0;
		my $best_eval = "NO_MATCH";
		my $best_header = "NO_MATCH";
		my $lineage_str_clean = "NO_MATCH";
		my $seqid_to_tax = 0;
		my $tax_level = "NO_MATCH";
		my $tax_acc = "NO_MATCH";
		my $source = "NO_MATCH";
		my $taxid = "NO_MATCH";
		my $rank = "NO_MATCH";
		my $num_preds = scalar((keys %{$contig_to_preds_ref->{$contig}}));
		foreach my $header (keys %{$contig_to_preds_ref->{$contig}})
		{
			my $curr_bitscore_tax = $contig_to_preds_ref->{$contig}->{$header}->{bitscore_tax};
			my $curr_eval_tax = $contig_to_preds_ref->{$contig}->{$header}->{evalue_tax};
			my $curr_lineage_str_clean = $contig_to_preds_ref->{$contig}->{$header}->{lineage_info};
			my $curr_seqid_to_tax = $contig_to_preds_ref->{$contig}->{$header}->{seqid_tax};
			my $curr_tax_level = tax_level_match($curr_seqid_to_tax);
			my $curr_tax_acc = $contig_to_preds_ref->{$contig}->{$header}->{acc_tax};
			my $curr_source = $contig_to_preds_ref->{$contig}->{$header}->{source};
			my $curr_taxid = $contig_to_preds_ref->{$contig}->{$header}->{taxid};
			my $curr_rank = $contig_to_preds_ref->{$contig}->{$header}->{rank};
			
			print $out_preds "$header\t$curr_bitscore_tax\t$curr_eval_tax\t$curr_source\t$curr_tax_acc\t$curr_seqid_to_tax\t$curr_tax_level\t$curr_lineage_str_clean\n";
			
			if ($curr_bitscore_tax > $best_bitscore)
			{
				$best_header = $header;
				$best_bitscore = $curr_bitscore_tax;
				$best_eval = $curr_eval_tax;
				$lineage_str_clean = $curr_lineage_str_clean;
				$seqid_to_tax = $curr_seqid_to_tax;
				$tax_level = $curr_tax_level;
				$tax_acc = $curr_tax_acc;
				$source = $curr_source;
				$taxid = $curr_taxid;
				$rank = $curr_rank;
			}
		}
		print $out_cont "$contig\t$best_header\t$best_bitscore\t$best_eval\t$source\t$tax_acc\t$seqid_to_tax\t$tax_level\t$lineage_str_clean\t$num_preds\n";
		print $out_corona "$num\t$taxid\t$rank\t$seqid_to_tax\t$contig\n";
		$num++;
	}
	close ($out_cont);
	close ($out_preds);
	close ($out_corona);
}

sub tax_level_match
{
	my ($seqid) = @_;
	if ($seqid > 0.95)
	{
		return "species";
	}
	if ($seqid > 0.8)
	{
		return "genus";
	}
	if ($seqid > 0.65)
	{
		return "family";
	}
	if ($seqid > 0.5)
	{
		return "order";
	}
	if ($seqid > 0.4)
	{
		return "class";
	}
	if ($seqid > 0.3)
	{
		return "phylum";
	}
	if ($seqid == 0)
	{
		return "NO_MATCH";
	}
	else
	{
		return "domain"
	}
}

sub add_best_hit_info
{
	my ($in_preds_best_hit_taxinfo_tsv, $source, $contig_to_preds_ref) = @_;

	open (my $in, "<", $in_preds_best_hit_taxinfo_tsv) or die "could not open $in_preds_best_hit_taxinfo_tsv for reading\n";
	my $line = <$in>;
	my $num_pred = 0;
	while (defined $line)
	{
		$num_pred++;
		chomp ($line);
		#A0A0E0CWD3_14989748|ERR599289_k119_1163103|-|181|9.062e-45|1|494|1270|1270[1270]:494[494]:777[777]	905079	no rank	Guillardia theta CCMP2712	Guillardia theta:Guillardia:Geminigeraceae:Pyrenomonadales:unknown:unknown:unknown:unknown:unknown:Eukaryota	1.074E-87	297	0.649	L1JFY6
		my ($header, $taxid, $rank, $name, $lineage_str, $evalue_tax, $bitscore_tax, $seqid_tax, $acc_tax) = split(/\t/, $line);
		if (($taxid != 0) and ($evalue_tax < 0.00001))
		{
			my ($target, $contig, $strand, $bitscore, $evalue) = split(/\|/, $header);
			my $lineage_str_clean = $lineage_str;
			#$lineage_str_clean =~ s/\:unknown//g;
			$lineage_str_clean =~ s/\s+/\_/g;
			
			if (exists $contig_to_preds_ref->{$contig})
			{
				if (exists $contig_to_preds_ref->{$contig}->{$header})
				{
					if ($contig_to_preds_ref->{$contig}->{$header}->{bitscore_tax} < $bitscore_tax)
					{
						$contig_to_preds_ref->{$contig}->{$header}->{lineage_info} = $lineage_str_clean;
						$contig_to_preds_ref->{$contig}->{$header}->{bitscore_tax} = $bitscore_tax;
						$contig_to_preds_ref->{$contig}->{$header}->{evalue_tax} = $evalue_tax;
						$contig_to_preds_ref->{$contig}->{$header}->{seqid_tax} = $seqid_tax;
						$contig_to_preds_ref->{$contig}->{$header}->{acc_tax} = $acc_tax;
						$contig_to_preds_ref->{$contig}->{$header}->{source} = $source;
						$contig_to_preds_ref->{$contig}->{$header}->{taxid} = $taxid;
						$contig_to_preds_ref->{$contig}->{$header}->{rank} = $rank;
					}
				}
				else
				{
					die "ERROR: '$header' not found in full hash\n";
				}
			}
			else
			{
				die "ERROR: '$header' not found in full hash\n";
			}
		}
		
		if ($num_pred % 10000 == 0)
		{
			print "processed prediction number $num_pred\n";
		}
		$line = <$in>;
	}
	close ($in);
}

sub parse_metaeuk_fasta
{
	my ($in_metaeuk_preds_fasta, $mmetsp_to_tax_info_ref, $contig_to_preds_ref) = @_;
	
	open (my $in, "<", $in_metaeuk_preds_fasta) or die "could not open $in_metaeuk_preds_fasta for reading";
	my $line = <$in>;
	my $num_pred = 0;
	while (defined $line)
	{
		chomp $line;
		if ($line =~ m/^>(.*)/)
		{
			$num_pred++;
			my $header = $1;
			my ($target, $contig, $strand, $bitscore, $evalue) = split(/\|/, $header);
			my $target_id_without_metaeuk_num = "";
			if ($target =~ m/^([a-zA-Z]+\d+)/)
			{
				$target_id_without_metaeuk_num = $1;
			}

			$contig_to_preds_ref->{$contig}->{$header}->{lineage_info} = "";
			$contig_to_preds_ref->{$contig}->{$header}->{bitscore_tax} = 0;
			$contig_to_preds_ref->{$contig}->{$header}->{evalue_tax} = "NA";
			$contig_to_preds_ref->{$contig}->{$header}->{seqid_tax} = 0;
			$contig_to_preds_ref->{$contig}->{$header}->{acc_tax} = "NA";
			$contig_to_preds_ref->{$contig}->{$header}->{source} = "no_match";
			
#			if (exists $mmetsp_to_tax_info_ref->{$target_id_without_metaeuk_num})
#			{
#				$contig_to_preds_ref->{$contig}->{$header}->{lineage_info} = $mmetsp_to_tax_info_ref->{$target_id_without_metaeuk_num};
#				$contig_to_preds_ref->{$contig}->{$header}->{bitscore_tax} = $bitscore;
#				$contig_to_preds_ref->{$contig}->{$header}->{evalue_tax} = $evalue;
#				$contig_to_preds_ref->{$contig}->{$header}->{source} = "SOURCE_MMETSP";
#			}
			if ($num_pred % 10000 == 0)
			{
				print "processed prediction number $num_pred\n";
			}
		}
		$line = <$in>;
	}
	close ($in);
	print "finished adding all metaeuk preds\n";
}


sub get_mmetsp_to_tax_info
{
	my ($in_mmetsp_to_taxinfo, $mmetsp_to_tax_info_ref) = @_;
	
	open (my $in, "<", $in_mmetsp_to_taxinfo) or die "could not open $in_mmetsp_to_taxinfo for reading\n";
	my @all_lines = <$in>;
	chomp (@all_lines);
	close ($in);
	
	shift @all_lines; # remove header
	#MMETSP_ID	name	taxid	lineage_str
	#MMETSP1473	Stichococcus	29646	Stichococcus|Microthamniales|Trebouxiophyceae|Chlorophyta|Viridiplantae|Eukaryota|cellular_organisms
	foreach my $info_line (@all_lines)
	{
		my ($mmetsp_id, $name, $taxid, $lineage_str) = split(/\t/, $info_line);
		my $lineage_str_colon = join(":",split(/\|/,$lineage_str));
		$mmetsp_to_tax_info_ref->{$mmetsp_id} = $lineage_str_colon;
		print "entered: '$mmetsp_id' with '$lineage_str_colon'\n";
	}
}
